@@ -1,6 +1,5 @@
 from authlib.common.security import generate_token
 from authlib.common.urls import url_decode
-from authlib.common.encoding import text_types
 from .rfc6749.parameters import (
     prepare_grant_uri,
     prepare_token_request,
@@ -10,6 +9,7 @@ from .rfc6749.parameters import (
 from .rfc7009 import prepare_revoke_token_request
 from .rfc7636 import create_s256_code_challenge
 from .auth import TokenAuth, ClientAuth
+from .base import OAuth2Error
 
 DEFAULT_HEADERS = {
     'Accept': 'application/json',
@@ -17,7 +17,7 @@ DEFAULT_HEADERS = {
 }
 
 
-class OAuth2Client(object):
+class OAuth2Client:
     """Construct a new OAuth 2 protocol client.
 
     :param session: Requests session object to communicate with
@@ -29,6 +29,7 @@ class OAuth2Client(object):
     :param revocation_endpoint_auth_method: client authentication method for
         revocation endpoint.
     :param scope: Scope that you needed to access user resources.
+    :param state: Shared secret to prevent CSRF attack.
     :param redirect_uri: Redirect URI you registered as callback.
     :param code_challenge_method: PKCE method name, only S256 is supported.
     :param token: A dict of token attributes such as ``access_token``,
@@ -37,9 +38,13 @@ class OAuth2Client(object):
         values: "header", "body", "uri".
     :param update_token: A function for you to update token. It accept a
         :class:`OAuth2Token` as parameter.
+    :param leeway: Time window in seconds before the actual expiration of the
+        authentication token, that the token is considered expired and will
+        be refreshed.
     """
     client_auth_class = ClientAuth
     token_auth_class = TokenAuth
+    oauth_error_class = OAuth2Error
 
     EXTRA_AUTHORIZE_PARAMS = (
         'response_mode', 'nonce', 'prompt', 'login_hint'
@@ -49,12 +54,14 @@ class OAuth2Client(object):
     def __init__(self, session, client_id=None, client_secret=None,
                  token_endpoint_auth_method=None,
                  revocation_endpoint_auth_method=None,
-                 scope=None, redirect_uri=None, code_challenge_method=None,
-                 token=None, token_placement='header', update_token=None, **metadata):
+                 scope=None, state=None, redirect_uri=None, code_challenge_method=None,
+                 token=None, token_placement='header', update_token=None, leeway=60,
+                 **metadata):
 
         self.session = session
         self.client_id = client_id
         self.client_secret = client_secret
+        self.state = state
 
         if token_endpoint_auth_method is None:
             if client_secret:
@@ -94,6 +101,8 @@ class OAuth2Client(object):
         }
         self._auth_methods = {}
 
+        self.leeway = leeway
+
     def register_client_auth_method(self, auth):
         """Extend client authenticate for token endpoint.
 
@@ -105,7 +114,7 @@ class OAuth2Client(object):
             self._auth_methods[auth.name] = auth
 
     def client_auth(self, auth_method):
-        if isinstance(auth_method, text_types) and auth_method in self._auth_methods:
+        if isinstance(auth_method, str) and auth_method in self._auth_methods:
             auth_method = self._auth_methods[auth_method]
         return self.client_auth_class(
             client_id=self.client_id,
@@ -155,7 +164,7 @@ class OAuth2Client(object):
         return uri, state
 
     def fetch_token(self, url=None, body='', method='POST', headers=None,
-                    auth=None, grant_type=None, **kwargs):
+                    auth=None, grant_type=None, state=None, **kwargs):
         """Generic method for fetching an access token from the token endpoint.
 
         :param url: Access Token endpoint URL, if not configured,
@@ -171,10 +180,11 @@ class OAuth2Client(object):
         :param grant_type: Use specified grant_type to fetch token
         :return: A :class:`OAuth2Token` object (a dict too).
         """
+        state = state or self.state
         # implicit  grant_type
         authorization_response = kwargs.pop('authorization_response', None)
         if authorization_response and '#' in authorization_response:
-            return self.token_from_fragment(authorization_response, kwargs.get('state'))
+            return self.token_from_fragment(authorization_response, state)
 
         session_kwargs = self._extract_session_request_params(kwargs)
 
@@ -182,13 +192,17 @@ class OAuth2Client(object):
             grant_type = 'authorization_code'
             params = parse_authorization_code_response(
                 authorization_response,
-                state=kwargs.get('state'),
+                state=state,
             )
             kwargs['code'] = params['code']
 
         if grant_type is None:
             grant_type = self.metadata.get('grant_type')
 
+        if grant_type is None:
+            grant_type = _guess_grant_type(kwargs)
+            self.metadata['grant_type'] = grant_type
+
         body = self._prepare_token_endpoint_body(body, grant_type, **kwargs)
 
         if auth is None:
@@ -205,31 +219,17 @@ class OAuth2Client(object):
             headers=headers, **session_kwargs
         )
 
-    def _fetch_token(self, url, body='', headers=None, auth=None,
-                     method='POST', **kwargs):
-        if method == 'GET':
-            if '?' in url:
-                url = '&'.join([url, body])
-            else:
-                url = '?'.join([url, body])
-            body = ''
-
-        if headers is None:
-            headers = DEFAULT_HEADERS
-
-        resp = self.session.request(
-            method, url, data=body, headers=headers, auth=auth, **kwargs)
-
-        for hook in self.compliance_hook['access_token_response']:
-            resp = hook(resp)
-
-        return self.parse_response_token(resp.json())
-
     def token_from_fragment(self, authorization_response, state=None):
         token = parse_implicit_response(authorization_response, state)
-        return self.parse_response_token(token)
+        if 'error' in token:
+            raise self.oauth_error_class(
+                error=token['error'],
+                description=token.get('error_description')
+            )
+        self.token = token
+        return token
 
-    def refresh_token(self, url, refresh_token=None, body='',
+    def refresh_token(self, url=None, refresh_token=None, body='',
                       auth=None, headers=None, **kwargs):
         """Fetch a new access token using a refresh token.
 
@@ -251,7 +251,10 @@ class OAuth2Client(object):
         )
 
         if headers is None:
-            headers = DEFAULT_HEADERS
+            headers = DEFAULT_HEADERS.copy()
+
+        if url is None:
+            url = self.metadata.get('token_endpoint')
 
         for hook in self.compliance_hook['refresh_token_request']:
             url, headers, body = hook(url, headers, body)
@@ -263,23 +266,22 @@ class OAuth2Client(object):
             url, refresh_token=refresh_token, body=body, headers=headers,
             auth=auth, **session_kwargs)
 
-    def _refresh_token(self, url, refresh_token=None, body='', headers=None,
-                       auth=None, **kwargs):
-        resp = self.session.post(
-            url, data=dict(url_decode(body)), headers=headers,
-            auth=auth, **kwargs)
-
-        for hook in self.compliance_hook['refresh_token_response']:
-            resp = hook(resp)
-
-        token = self.parse_response_token(resp.json())
-        if 'refresh_token' not in token:
-            self.token['refresh_token'] = refresh_token
-
-        if callable(self.update_token):
-            self.update_token(self.token, refresh_token=refresh_token)
-
-        return self.token
+    def ensure_active_token(self, token=None):
+        if token is None:
+            token = self.token
+        if not token.is_expired(leeway=self.leeway):
+            return True
+        refresh_token = token.get('refresh_token')
+        url = self.metadata.get('token_endpoint')
+        if refresh_token and url:
+            self.refresh_token(url, refresh_token=refresh_token)
+            return True
+        elif self.metadata.get('grant_type') == 'client_credentials':
+            access_token = token['access_token']
+            new_token = self.fetch_token(url, grant_type='client_credentials')
+            if self.update_token:
+                self.update_token(new_token, access_token=access_token)
+            return True
 
     def revoke_token(self, url, token=None, token_type_hint=None,
                      body=None, auth=None, headers=None, **kwargs):
@@ -323,32 +325,6 @@ class OAuth2Client(object):
             token=token, token_type_hint=token_type_hint,
             body=body, auth=auth, headers=headers, **kwargs)
 
-    def _handle_token_hint(self, hook, url, token=None, token_type_hint=None,
-                           body=None, auth=None, headers=None, **kwargs):
-        if token is None and self.token:
-            token = self.token.get('refresh_token') or self.token.get('access_token')
-
-        if body is None:
-            body = ''
-
-        body, headers = prepare_revoke_token_request(
-            token, token_type_hint, body, headers)
-
-        for hook in self.compliance_hook[hook]:
-            url, headers, body = hook(url, headers, body)
-
-        if auth is None:
-            auth = self.client_auth(self.revocation_endpoint_auth_method)
-
-        session_kwargs = self._extract_session_request_params(kwargs)
-        return self._http_post(
-            url, body, auth=auth, headers=headers, **session_kwargs)
-
-    def _http_post(self, url, body=None, auth=None, headers=None, **kwargs):
-        return self.session.post(
-            url, data=dict(url_decode(body)),
-            headers=headers, auth=auth, **kwargs)
-
     def register_compliance_hook(self, hook_type, hook):
         """Register a hook for request/response tweaking.
 
@@ -370,19 +346,76 @@ class OAuth2Client(object):
                              hook_type, self.compliance_hook)
         self.compliance_hook[hook_type].add(hook)
 
-    def parse_response_token(self, token):
-        if 'error' not in token:
-            self.token = token
-            return self.token
+    def parse_response_token(self, resp):
+        if resp.status_code >= 500:
+            resp.raise_for_status()
 
-        error = token['error']
-        description = token.get('error_description', error)
-        self.handle_error(error, description)
+        token = resp.json()
+        if 'error' in token:
+            raise self.oauth_error_class(
+                error=token['error'],
+                description=token.get('error_description')
+            )
+        self.token = token
+        return self.token
+
+    def _fetch_token(self, url, body='', headers=None, auth=None,
+                     method='POST', **kwargs):
+
+        if method.upper() == 'POST':
+            resp = self.session.post(
+                url, data=dict(url_decode(body)),
+                headers=headers, auth=auth, **kwargs)
+        else:
+            if '?' in url:
+                url = '&'.join([url, body])
+            else:
+                url = '?'.join([url, body])
+            resp = self.session.request(method, url, headers=headers, auth=auth, **kwargs)
+
+        for hook in self.compliance_hook['access_token_response']:
+            resp = hook(resp)
+
+        return self.parse_response_token(resp)
+
+    def _refresh_token(self, url, refresh_token=None, body='', headers=None,
+                       auth=None, **kwargs):
+        resp = self._http_post(url, body=body, auth=auth, headers=headers, **kwargs)
+
+        for hook in self.compliance_hook['refresh_token_response']:
+            resp = hook(resp)
+
+        token = self.parse_response_token(resp)
+        if 'refresh_token' not in token:
+            self.token['refresh_token'] = refresh_token
+
+        if callable(self.update_token):
+            self.update_token(self.token, refresh_token=refresh_token)
+
+        return self.token
+
+    def _handle_token_hint(self, hook, url, token=None, token_type_hint=None,
+                           body=None, auth=None, headers=None, **kwargs):
+        if token is None and self.token:
+            token = self.token.get('refresh_token') or self.token.get('access_token')
+
+        if body is None:
+            body = ''
+
+        body, headers = prepare_revoke_token_request(
+            token, token_type_hint, body, headers)
+
+        for hook in self.compliance_hook[hook]:
+            url, headers, body = hook(url, headers, body)
+
+        if auth is None:
+            auth = self.client_auth(self.revocation_endpoint_auth_method)
+
+        session_kwargs = self._extract_session_request_params(kwargs)
+        return self._http_post(
+            url, body, auth=auth, headers=headers, **session_kwargs)
 
     def _prepare_token_endpoint_body(self, body, grant_type, **kwargs):
-        if grant_type is None:
-            grant_type = _guess_grant_type(kwargs)
-
         if grant_type == 'authorization_code':
             if 'redirect_uri' not in kwargs:
                 kwargs['redirect_uri'] = self.redirect_uri
@@ -400,9 +433,13 @@ class OAuth2Client(object):
                 rv[k] = kwargs.pop(k)
         return rv
 
-    @staticmethod
-    def handle_error(error_type, error_description):
-        raise ValueError('{}: {}'.format(error_type, error_description))
+    def _http_post(self, url, body=None, auth=None, headers=None, **kwargs):
+        return self.session.post(
+            url, data=dict(url_decode(body)),
+            headers=headers, auth=auth, **kwargs)
+
+    def __del__(self):
+        del self.session
 
 
 def _guess_grant_type(kwargs):
