@@ -52,12 +52,12 @@ async def assert_token_in_uri(request):
     ]
 )
 async def test_add_token_get_request(assert_func, token_placement):
-    mock_response = AsyncMockDispatch({'a': 'a'}, assert_func=assert_func)
+    transport = ASGITransport(AsyncMockDispatch({'a': 'a'}, assert_func=assert_func))
     async with AsyncOAuth2Client(
             'foo',
             token=default_token,
             token_placement=token_placement,
-            app=mock_response
+            transport=transport
     ) as client:
         resp = await client.get('https://i.b')
 
@@ -75,12 +75,12 @@ async def test_add_token_get_request(assert_func, token_placement):
     ]
 )
 async def test_add_token_to_streaming_request(assert_func, token_placement):
-    mock_response = AsyncMockDispatch({'a': 'a'}, assert_func=assert_func)
+    transport = ASGITransport(AsyncMockDispatch({'a': 'a'}, assert_func=assert_func))
     async with AsyncOAuth2Client(
             'foo',
             token=default_token,
             token_placement=token_placement,
-            app=mock_response
+            transport=transport
     ) as client:
         async with client.stream("GET", 'https://i.b') as stream:
             await stream.aread()
@@ -94,9 +94,9 @@ async def test_add_token_to_streaming_request(assert_func, token_placement):
         'foo',
         token=default_token,
         token_placement="header",
-        app=AsyncMockDispatch({'a': 'a'}, assert_func=assert_token_in_header)
+        transport=ASGITransport(AsyncMockDispatch({'a': 'a'}, assert_func=assert_token_in_header)),
     ),
-    AsyncClient(transport=ASGITransport(app=AsyncMockDispatch({'a': 'a'})))
+    AsyncClient(transport=ASGITransport(AsyncMockDispatch({'a': 'a'})))
 ])
 async def test_httpx_client_stream_match(client):
     async with client as client_entered:
@@ -151,21 +151,21 @@ async def test_fetch_token_post():
         assert 'client_id=' in content
         assert 'grant_type=authorization_code' in content
 
-    mock_response = AsyncMockDispatch(default_token, assert_func=assert_func)
-    async with AsyncOAuth2Client('foo', app=mock_response) as client:
+    transport = ASGITransport(AsyncMockDispatch(default_token, assert_func=assert_func))
+    async with AsyncOAuth2Client('foo', transport=transport) as client:
         token = await client.fetch_token(url, authorization_response='https://i.b/?code=v')
         assert token == default_token
 
     async with AsyncOAuth2Client(
             'foo',
             token_endpoint_auth_method='none',
-            app=mock_response
+            transport=transport
     ) as client:
         token = await client.fetch_token(url, code='v')
         assert token == default_token
 
-    mock_response = AsyncMockDispatch({'error': 'invalid_request'})
-    async with AsyncOAuth2Client('foo', app=mock_response) as client:
+    transport = ASGITransport(AsyncMockDispatch({'error': 'invalid_request'}))
+    async with AsyncOAuth2Client('foo', transport=transport) as client:
         with pytest.raises(OAuthError):
             await client.fetch_token(url)
 
@@ -180,8 +180,8 @@ async def test_fetch_token_get():
         assert 'client_id=' in url
         assert 'grant_type=authorization_code' in url
 
-    mock_response = AsyncMockDispatch(default_token, assert_func=assert_func)
-    async with AsyncOAuth2Client('foo', app=mock_response) as client:
+    transport = ASGITransport(AsyncMockDispatch(default_token, assert_func=assert_func))
+    async with AsyncOAuth2Client('foo', transport=transport) as client:
         authorization_response = 'https://i.b/?code=v'
         token = await client.fetch_token(
             url, authorization_response=authorization_response, method='GET')
@@ -190,7 +190,7 @@ async def test_fetch_token_get():
     async with AsyncOAuth2Client(
             'foo',
             token_endpoint_auth_method='none',
-            app=mock_response
+            transport=transport
     ) as client:
         token = await client.fetch_token(url, code='v', method='GET')
         assert token == default_token
@@ -211,11 +211,11 @@ async def test_token_auth_method_client_secret_post():
         assert 'client_secret=bar' in content
         assert 'grant_type=authorization_code' in content
 
-    mock_response = AsyncMockDispatch(default_token, assert_func=assert_func)
+    transport = ASGITransport(AsyncMockDispatch(default_token, assert_func=assert_func))
     async with AsyncOAuth2Client(
             'foo', 'bar',
             token_endpoint_auth_method='client_secret_post',
-            app=mock_response
+            transport=transport
     ) as client:
         token = await client.fetch_token(url, code='v')
 
@@ -231,8 +231,8 @@ async def test_access_token_response_hook():
         return resp
 
     access_token_response_hook = mock.Mock(side_effect=_access_token_response_hook)
-    app = AsyncMockDispatch(default_token)
-    async with AsyncOAuth2Client('foo', token=default_token, app=app) as sess:
+    transport = ASGITransport(AsyncMockDispatch(default_token))
+    async with AsyncOAuth2Client('foo', token=default_token, transport=transport) as sess:
         sess.register_compliance_hook(
             'access_token_response',
             access_token_response_hook
@@ -252,8 +252,8 @@ async def test_password_grant_type():
         assert 'scope=profile' in content
         assert 'grant_type=password' in content
 
-    app = AsyncMockDispatch(default_token, assert_func=assert_func)
-    async with AsyncOAuth2Client('foo', scope='profile', app=app) as sess:
+    transport = ASGITransport(AsyncMockDispatch(default_token, assert_func=assert_func))
+    async with AsyncOAuth2Client('foo', scope='profile', transport=transport) as sess:
         token = await sess.fetch_token(url, username='v', password='v')
         assert token == default_token
 
@@ -272,8 +272,8 @@ async def test_client_credentials_type():
         assert 'scope=profile' in content
         assert 'grant_type=client_credentials' in content
 
-    app = AsyncMockDispatch(default_token, assert_func=assert_func)
-    async with AsyncOAuth2Client('foo', scope='profile', app=app) as sess:
+    transport = ASGITransport(AsyncMockDispatch(default_token, assert_func=assert_func))
+    async with AsyncOAuth2Client('foo', scope='profile', transport=transport) as sess:
         token = await sess.fetch_token(url)
         assert token == default_token
 
@@ -290,9 +290,9 @@ async def test_cleans_previous_token_before_fetching_new_one():
     new_token['expires_at'] = now + 3600
     url = 'https://example.com/token'
 
-    app = AsyncMockDispatch(new_token)
+    transport = ASGITransport(AsyncMockDispatch(new_token))
     with mock.patch('time.time', lambda: now):
-        async with AsyncOAuth2Client('foo', token=default_token, app=app) as sess:
+        async with AsyncOAuth2Client('foo', token=default_token, transport=transport) as sess:
             assert await sess.fetch_token(url) == new_token
 
 
@@ -316,10 +316,10 @@ async def test_auto_refresh_token():
         token_type='bearer', expires_at=100
     )
 
-    app = AsyncMockDispatch(default_token)
+    transport = ASGITransport(AsyncMockDispatch(default_token))
     async with AsyncOAuth2Client(
             'foo', token=old_token, token_endpoint='https://i.b/token',
-            update_token=update_token, app=app
+            update_token=update_token, transport=transport
     ) as sess:
         await sess.get('https://i.b/user')
         assert update_token.called is True
@@ -331,7 +331,7 @@ async def test_auto_refresh_token():
     )
     async with AsyncOAuth2Client(
             'foo', token=old_token, token_endpoint='https://i.b/token',
-            update_token=update_token, app=app
+            update_token=update_token, transport=transport
     ) as sess:
         with pytest.raises(OAuthError):
             await sess.get('https://i.b/user')
@@ -352,13 +352,13 @@ async def test_auto_refresh_token2():
         expires_at=100
     )
 
-    app = AsyncMockDispatch(default_token)
+    transport = ASGITransport(AsyncMockDispatch(default_token))
 
     async with AsyncOAuth2Client(
             'foo', token=old_token,
             token_endpoint='https://i.b/token',
             grant_type='client_credentials',
-            app=app,
+            transport=transport,
     ) as client:
         await client.get('https://i.b/user')
         assert update_token.called is False
@@ -366,7 +366,7 @@ async def test_auto_refresh_token2():
     async with AsyncOAuth2Client(
             'foo', token=old_token, token_endpoint='https://i.b/token',
             update_token=update_token, grant_type='client_credentials',
-            app=app,
+            transport=transport,
     ) as client:
         await client.get('https://i.b/user')
         assert update_token.called is True
@@ -386,12 +386,12 @@ async def test_auto_refresh_token3():
         expires_at=100
     )
 
-    app = AsyncMockDispatch(default_token)
+    transport = ASGITransport(AsyncMockDispatch(default_token))
 
     async with AsyncOAuth2Client(
             'foo', token=old_token, token_endpoint='https://i.b/token',
             update_token=update_token, grant_type='client_credentials',
-            app=app,
+            transport=transport,
     ) as client:
         await client.post('https://i.b/user', json={'foo': 'bar'})
         assert update_token.called is True
@@ -412,12 +412,12 @@ async def test_auto_refresh_token4():
         expires_at=100
     )
 
-    app = AsyncMockDispatch(default_token)
+    transport = ASGITransport(AsyncMockDispatch(default_token))
 
     async with AsyncOAuth2Client(
             'foo', token=old_token, token_endpoint='https://i.b/token',
             update_token=update_token, grant_type='client_credentials',
-            app=app,
+            transport=transport,
     ) as client:
         coroutines = [client.get('https://i.b/user') for x in range(10)]
         await asyncio.gather(*coroutines)
@@ -426,9 +426,9 @@ async def test_auto_refresh_token4():
 @pytest.mark.asyncio
 async def test_revoke_token():
     answer = {'status': 'ok'}
-    app = AsyncMockDispatch(answer)
+    transport = ASGITransport(AsyncMockDispatch(answer))
 
-    async with AsyncOAuth2Client('a', app=app) as sess:
+    async with AsyncOAuth2Client('a', transport=transport) as sess:
         resp = await sess.revoke_token('https://i.b/token', 'hi')
         assert resp.json() == answer
 
@@ -441,6 +441,7 @@ async def test_revoke_token():
 
 @pytest.mark.asyncio
 async def test_request_without_token():
-    async with AsyncOAuth2Client('a', app=AsyncMockDispatch()) as client:
+    transport = ASGITransport(AsyncMockDispatch())
+    async with AsyncOAuth2Client('a', transport=transport) as client:
         with pytest.raises(OAuthError):
             await client.get('https://i.b/token')
